@@ -40,8 +40,6 @@ public:
   TextParameter(QObject * parent = 0);
   ~TextParameter();
   void addTo(QWidget *, int row) override;
-  void addToKeypointList(KeypointList &) const override;
-  void extractPositionFromKeypointList(KeypointList &) override;
   QString textValue() const override;
   QString unquotedTextValue() const override;
   void setValue(const QString & value) override;
