@@ -44,6 +44,7 @@
 #include "FilterSelector/FiltersVisibilityMap.h"
 #include "Globals.h"
 #include "GmicStdlib.h"
+#include "IconLoader.h"
 #include "LayersExtentProxy.h"
 #include "Logger.h"
 #include "ParametersCache.h"
@@ -70,15 +71,10 @@ MainWindow::MainWindow(QWidget * parent) : QWidget(parent), ui(new Ui::MainWindo
   tsp.append(QString("/usr/share/icons/gnome"));
   QIcon::setThemeSearchPaths(tsp);
 
-  _filterUpdateWidgets = {ui->previewWidget,   ui->tbZoomIn,     ui->tbZoomOut,  ui->tbZoomReset,  ui->zoomLevelSelector, ui->filtersView,      ui->filterParams,
-                          ui->tbUpdateFilters, ui->pbFullscreen, ui->pbSettings, ui->pbOk,         ui->pbApply,           ui->inOutSelector,    ui->tbResetParameters,
-                          ui->searchField,     ui->cbPreview,    ui->tbAddFave,  ui->tbRemoveFave, ui->tbRenameFave,      ui->tbExpandCollapse, ui->tbSelectionMode};
+  _filterUpdateWidgets = {ui->previewWidget, ui->zoomLevelSelector, ui->filtersView,      ui->filterParams,      ui->tbUpdateFilters, ui->pbFullscreen, ui->pbSettings,
+                          ui->pbOk,          ui->pbApply,           ui->inOutSelector,    ui->tbResetParameters, ui->searchField,     ui->cbPreview,    ui->tbAddFave,
+                          ui->tbRemoveFave,  ui->tbRenameFave,      ui->tbExpandCollapse, ui->tbSelectionMode};
 
-  ui->tbZoomIn->setToolTip(tr("Zoom in"));
-  ui->tbZoomOut->setToolTip(tr("Zoom out"));
-  ui->tbZoomReset->setToolTip(tr("Reset zoom"));
-
-  ui->labelWarning->setToolTip(tr("Warning: Preview may be inaccurate (zoom factor has been modified)"));
   ui->tbAddFave->setToolTip(tr("Add fave"));
 
   ui->tbResetParameters->setToolTip(tr("Reset parameters to default values"));
@@ -116,6 +112,8 @@ MainWindow::MainWindow(QWidget * parent) : QWidget(parent), ui(new Ui::MainWindo
   ui->inOutSelector->disable();
   ui->splitter->setChildrenCollapsible(false);
 
+  ui->zoomLevelSelector->setPreviewWidget(ui->previewWidget);
+
   QAction * searchAction = new QAction(this);
   searchAction->setShortcut(QKeySequence::Find);
   searchAction->setShortcutContext(Qt::ApplicationShortcut);
@@ -159,10 +157,9 @@ MainWindow::MainWindow(QWidget * parent) : QWidget(parent), ui(new Ui::MainWindo
   connect(escAction, SIGNAL(triggered(bool)), this, SLOT(onEscapeKeyPressed()));
   addAction(escAction);
 
-  LayersExtentProxy::clearCache();
+  LayersExtentProxy::clear();
   QSize layersExtent = LayersExtentProxy::getExtent(ui->inOutSelector->inputMode());
   ui->previewWidget->setFullImageSize(layersExtent);
-
   _lastPreviewKeypointBurstUpdateTime = 0;
 
   TIMING;
@@ -186,13 +183,10 @@ MainWindow::~MainWindow()
 
 void MainWindow::setIcons()
 {
-  ui->tbRenameFave->setIcon(QIcon(":/resources/rename.png"));
+  ui->tbRenameFave->setIcon(LOAD_ICON("rename"));
   ui->pbSettings->setIcon(LOAD_ICON("package_settings"));
   ui->pbFullscreen->setIcon(LOAD_ICON("view-fullscreen"));
-  ui->tbZoomOut->setIcon(LOAD_ICON("zoom-out"));
-  ui->tbZoomIn->setIcon(LOAD_ICON("zoom-in"));
-  ui->tbZoomReset->setIcon(LOAD_ICON("view-refresh"));
-  ui->tbUpdateFilters->setIcon(LOAD_ICON("view-refresh"));
+  ui->tbUpdateFilters->setIcon(LOAD_ICON_NO_DARKENED("view-refresh"));
   ui->pbApply->setIcon(LOAD_ICON("system-run"));
   ui->pbOk->setIcon(LOAD_ICON("insert-image"));
   ui->tbResetParameters->setIcon(LOAD_ICON("view-refresh"));
@@ -204,8 +198,6 @@ void MainWindow::setIcons()
   _collapseIcon = LOAD_ICON("draw-arrow-up");
   _expandCollapseIcon = &_expandIcon;
   ui->tbExpandCollapse->setIcon(_expandIcon);
-
-  ui->labelWarning->setPixmap(QPixmap(":/images/no_warning.png"));
 }
 
 void MainWindow::setDarkTheme()
@@ -224,11 +216,15 @@ void MainWindow::setDarkTheme()
   p.setColor(QPalette::Link, linkColor);
   p.setColor(QPalette::LinkVisited, linkColor);
 
-  p.setColor(QPalette::Disabled, QPalette::Button, QColor(53, 53, 53));
-  p.setColor(QPalette::Disabled, QPalette::Window, QColor(53, 53, 53));
-  p.setColor(QPalette::Disabled, QPalette::Text, QColor(110, 110, 110));
-  p.setColor(QPalette::Disabled, QPalette::ButtonText, QColor(110, 110, 110));
-  p.setColor(QPalette::Disabled, QPalette::WindowText, QColor(110, 110, 110));
+  const QColor disabledGray(40, 40, 40);
+  const QColor disabledTextGray(128, 128, 128);
+  p.setColor(QPalette::Disabled, QPalette::Window, disabledGray);
+  p.setColor(QPalette::Disabled, QPalette::Base, disabledGray);
+  p.setColor(QPalette::Disabled, QPalette::AlternateBase, disabledGray);
+  p.setColor(QPalette::Disabled, QPalette::Button, disabledGray);
+  p.setColor(QPalette::Disabled, QPalette::Text, disabledTextGray);
+  p.setColor(QPalette::Disabled, QPalette::ButtonText, disabledTextGray);
+  p.setColor(QPalette::Disabled, QPalette::WindowText, disabledTextGray);
   qApp->setPalette(p);
 
   p = ui->cbInternetUpdate->palette();
@@ -250,6 +246,7 @@ void MainWindow::setDarkTheme()
                       "QGroupBox { border: 1px solid #808080; margin-top: 4ex; } "
                       "QFileDialog QAbstractItemView { background: #505050; } "
                       "QComboBox:editable { background: #505050; } "
+                      "QComboBox::disabled { background: rgb(40,40,40); } "
                       "QProgressBar { background: #505050; }";
   qApp->setStyleSheet(css);
   ui->inOutSelector->setDarkTheme();
@@ -344,7 +341,7 @@ void MainWindow::onStartupFiltersUpdateFinished(int status)
 
   // Let the standalone version load an image, if necessary (not pretty)
   if (GmicQt::HostApplicationName.isEmpty()) {
-    LayersExtentProxy::clearCache();
+    LayersExtentProxy::clear();
     QSize extent = LayersExtentProxy::getExtent(ui->inOutSelector->inputMode());
     ui->previewWidget->setFullImageSize(extent);
     ui->previewWidget->update();
@@ -372,24 +369,10 @@ void MainWindow::onStartupFiltersUpdateFinished(int status)
   // after the very first resize event).
 }
 
-void MainWindow::onZoomIn()
-{
-  ui->previewWidget->zoomIn();
-}
-
-void MainWindow::onZoomOut()
-{
-  ui->previewWidget->zoomOut();
-}
-
 void MainWindow::showZoomWarningIfNeeded()
 {
   const FiltersPresenter::Filter & currentFilter = _filtersPresenter->currentFilter();
-  if (!currentFilter.hash.isEmpty() && !currentFilter.isAccurateIfZoomed && !ui->previewWidget->isAtDefaultZoom()) {
-    ui->labelWarning->setPixmap(QPixmap(":/images/warning.png"));
-  } else {
-    ui->labelWarning->setPixmap(QPixmap(":/images/no_warning.png"));
-  }
+  ui->zoomLevelSelector->showWarning(!currentFilter.hash.isEmpty() && !currentFilter.isAccurateIfZoomed && !ui->previewWidget->isAtDefaultZoom());
 }
 
 void MainWindow::updateZoomLabel(double zoom)
@@ -495,9 +478,9 @@ void MainWindow::makeConnections()
   connect(ui->previewWidget, SIGNAL(previewUpdateRequested()), this, SLOT(onPreviewUpdateRequested()));
   connect(ui->previewWidget, SIGNAL(keypointPositionsChanged(unsigned int, unsigned long)), this, SLOT(onPreviewKeypointsEvent(unsigned int, unsigned long)));
 
-  connect(ui->tbZoomIn, SIGNAL(clicked(bool)), this, SLOT(onZoomIn()));
-  connect(ui->tbZoomOut, SIGNAL(clicked(bool)), this, SLOT(onZoomOut()));
-  connect(ui->tbZoomReset, SIGNAL(clicked(bool)), this, SLOT(onPreviewZoomReset()));
+  connect(ui->zoomLevelSelector, SIGNAL(zoomIn()), ui->previewWidget, SLOT(zoomIn()));
+  connect(ui->zoomLevelSelector, SIGNAL(zoomOut()), ui->previewWidget, SLOT(zoomOut()));
+  connect(ui->zoomLevelSelector, SIGNAL(zoomReset()), this, SLOT(onPreviewZoomReset()));
 
   connect(ui->tbAddFave, SIGNAL(clicked(bool)), this, SLOT(onAddFave()));
   connect(_filtersPresenter, SIGNAL(faveAdditionRequested(QString)), this, SLOT(onAddFave()));
@@ -520,6 +503,7 @@ void MainWindow::makeConnections()
   connect(&_processor, SIGNAL(previewCommandFailed(QString)), this, SLOT(onPreviewError(QString)));
   connect(&_processor, SIGNAL(fullImageProcessingFailed(QString)), this, SLOT(onFullImageProcessingError(QString)));
   connect(&_processor, SIGNAL(fullImageProcessingDone()), this, SLOT(onFullImageProcessingDone()));
+  connect(&_processor, SIGNAL(aboutToSendImagesToHost()), ui->progressInfoWidget, SLOT(stopAnimationAndHide()));
 }
 
 void MainWindow::onPreviewUpdateRequested()
@@ -665,6 +649,22 @@ void MainWindow::onInputModeChanged(GmicQt::InputMode mode)
   ui->previewWidget->sendUpdateRequest();
 }
 
+void MainWindow::setZoomConstraint()
+{
+  const FiltersPresenter::Filter & currentFilter = _filtersPresenter->currentFilter();
+  ZoomConstraint constraint;
+  if (currentFilter.hash.isEmpty() || currentFilter.isAccurateIfZoomed || DialogSettings::previewZoomAlwaysEnabled() || (currentFilter.previewFactor == GmicQt::PreviewFactorAny)) {
+    constraint = ZoomConstraint::Any;
+  } else if (currentFilter.previewFactor == GmicQt::PreviewFactorActualSize) {
+    constraint = ZoomConstraint::OneOrMore;
+  } else {
+    constraint = ZoomConstraint::Fixed;
+  }
+  showZoomWarningIfNeeded();
+  ui->zoomLevelSelector->setZoomConstraint(constraint);
+  ui->previewWidget->setZoomConstraint(constraint);
+}
+
 void MainWindow::onFullImageProcessingDone()
 {
   ui->progressInfoWidget->stopAnimationAndHide();
@@ -676,7 +676,7 @@ void MainWindow::onFullImageProcessingDone()
   if ((_pendingActionAfterCurrentProcessing == OkAction || _pendingActionAfterCurrentProcessing == CloseAction)) {
     close();
   } else {
-    LayersExtentProxy::clearCache();
+    // Extent cache has been cleared by the GmicProcessor
     QSize extent = LayersExtentProxy::getExtent(ui->inOutSelector->inputMode());
     ui->previewWidget->updateFullImageSizeIfDifferent(extent); // FIXME: updateIfDifferent ?
     ui->previewWidget->sendUpdateRequest();
@@ -772,7 +772,7 @@ void MainWindow::onPreviewZoomReset()
   if (!_filtersPresenter->currentFilter().hash.isEmpty()) {
     ui->previewWidget->setPreviewFactor(_filtersPresenter->currentFilter().previewFactor, true);
     ui->previewWidget->sendUpdateRequest();
-    ui->labelWarning->setPixmap(QPixmap(":/images/no_warning.png"));
+    ui->zoomLevelSelector->showWarning(false);
   }
 }
 
@@ -836,7 +836,6 @@ void MainWindow::saveSettings()
 void MainWindow::loadSettings()
 {
   QSettings settings;
-  DialogSettings::loadSettings();
   _filtersPresenter->loadSettings(settings);
 
   _lastExecutionOK = settings.value("LastExecution/ExitedNormally", true).toBool();
@@ -1002,7 +1001,7 @@ void MainWindow::activateFilter(bool resetZoom)
     ui->filterName->setVisible(true);
     ui->tbAddFave->setEnabled(true);
     ui->previewWidget->setPreviewFactor(filter.previewFactor, resetZoom);
-    showZoomWarningIfNeeded();
+    setZoomConstraint();
     _okButtonShouldApply = true;
     ui->tbResetParameters->setVisible(true);
     ui->tbRemoveFave->setEnabled(filter.isAFave);
@@ -1020,7 +1019,7 @@ void MainWindow::setNoFilter()
   ui->filterName->setVisible(false);
   ui->tbAddFave->setEnabled(false);
   ui->tbResetParameters->setVisible(false);
-  ui->labelWarning->setPixmap(QPixmap(":/images/no_warning.png"));
+  ui->zoomLevelSelector->showWarning(false);
   _okButtonShouldApply = false;
 
   ui->tbRemoveFave->setEnabled(false);
@@ -1161,6 +1160,19 @@ void MainWindow::onSettingsClicked()
   if (shouldUpdatePreview) {
     ui->previewWidget->sendUpdateRequest();
   }
+  // Manage zoom constraints
+  setZoomConstraint();
+  if (!DialogSettings::previewZoomAlwaysEnabled()) {
+    const FiltersPresenter::Filter & filter = _filtersPresenter->currentFilter();
+    if (((ui->previewWidget->zoomConstraint() == ZoomConstraint::Fixed) && (ui->previewWidget->defaultZoomFactor() != ui->previewWidget->currentZoomFactor())) ||
+        ((ui->previewWidget->zoomConstraint() == ZoomConstraint::OneOrMore) && (ui->previewWidget->currentZoomFactor() < 1.0))) {
+      ui->previewWidget->setPreviewFactor(filter.previewFactor, true);
+      if (ui->cbPreview->isChecked()) {
+        ui->previewWidget->sendUpdateRequest();
+      }
+    }
+  }
+  showZoomWarningIfNeeded();
 }
 
 bool MainWindow::confirmAbortProcessingOnCloseRequest()
