@@ -32,6 +32,7 @@
 #include <algorithm>
 #include <functional>
 #include "Common.h"
+#include "CroppedActiveLayerProxy.h"
 #include "DialogSettings.h"
 #include "Globals.h"
 #include "GmicStdlib.h"
@@ -45,7 +46,7 @@
 
 const PreviewWidget::PreviewRect PreviewWidget::PreviewRect::Full{0.0, 0.0, 1.0, 1.0};
 
-PreviewWidget::PreviewWidget(QWidget * parent) : QWidget(parent), _cachedOriginalImage(new gmic_image<float>())
+PreviewWidget::PreviewWidget(QWidget * parent) : QWidget(parent)
 {
   setAutoFillBackground(false);
   _image = new cimg_library::CImg<float>;
@@ -57,11 +58,10 @@ PreviewWidget::PreviewWidget(QWidget * parent) : QWidget(parent), _cachedOrigina
   _visibleRect = PreviewRect::Full;
   saveVisibleCenter();
 
-  _cachedOriginalImagePosition = {-1.0, -1.0, -1.0, -1.0};
-
   _pendingResize = false;
   _previewEnabled = true;
   _currentZoomFactor = 1.0;
+  _zoomConstraint = ZoomConstraint::Any;
   _timerID = 0;
   _savedPreviewIsValid = false;
   _paintOriginalImage = true;
@@ -131,7 +131,7 @@ void PreviewWidget::setPreviewErrorMessage(const QString & message)
 void PreviewWidget::setFullImageSize(const QSize & size)
 {
   _fullImageSize = size;
-  _cachedOriginalImagePosition = {-1.0, -1.0, -1.0, -1.0};
+  CroppedActiveLayerProxy::clear();
   updateVisibleRect();
   saveVisibleCenter();
 }
@@ -141,8 +141,7 @@ void PreviewWidget::updateFullImageSizeIfDifferent(const QSize & size)
   if (size != _fullImageSize) {
     setFullImageSize(size);
   } else {
-    // Make sure the cached image is no longer used, even if its size is the same
-    _cachedOriginalImagePosition = {-1.0, -1.0, -1.0, -1.0};
+    CroppedActiveLayerProxy::clear();
   }
 }
 
@@ -478,7 +477,6 @@ void PreviewWidget::leaveEvent(QEvent *)
 void PreviewWidget::wheelEvent(QWheelEvent * event)
 {
   double degrees = event->angleDelta().y() / 8.0;
-  // int steps = static_cast<int>(std::fabs(degrees) / 15.0);
   int steps = static_cast<int>(std::fabs(degrees) / 15.0);
   if (degrees > 0.0) {
     zoomIn(event->pos() - _imagePosition.topLeft(), steps);
@@ -687,7 +685,7 @@ void PreviewWidget::zoomFullImage()
 
 void PreviewWidget::zoomIn(QPoint p, int steps)
 {
-  if (_fullImageSize.isNull()) {
+  if (_fullImageSize.isNull() || (_zoomConstraint == ZoomConstraint::Fixed)) {
     return;
   }
   double previousZoomFactor = _currentZoomFactor;
@@ -716,7 +714,7 @@ void PreviewWidget::zoomIn(QPoint p, int steps)
 
 void PreviewWidget::zoomOut(QPoint p, int steps)
 {
-  if (isAtFullZoom() || _fullImageSize.isNull()) {
+  if ((_zoomConstraint == ZoomConstraint::Fixed) || ((_zoomConstraint == ZoomConstraint::OneOrMore) && (_currentZoomFactor <= 1.0)) || isAtFullZoom() || _fullImageSize.isNull()) {
     return;
   }
   double mouseX = p.x() / (_currentZoomFactor * _fullImageSize.width()) + _visibleRect.x;
@@ -724,6 +722,9 @@ void PreviewWidget::zoomOut(QPoint p, int steps)
   while (steps--) {
     _currentZoomFactor /= 1.2;
   }
+  if ((_zoomConstraint == ZoomConstraint::OneOrMore) && (_currentZoomFactor <= 1.0)) {
+    _currentZoomFactor = 1.0;
+  }
   updateVisibleRect();
   if (isAtFullZoom()) {
     _currentZoomFactor = std::min(width() / (double)_fullImageSize.width(), height() / (double)_fullImageSize.height());
@@ -741,6 +742,9 @@ void PreviewWidget::setZoomLevel(double zoom)
   if ((zoom == _currentZoomFactor) || _fullImageSize.isNull()) {
     return;
   }
+  if ((_zoomConstraint == ZoomConstraint::OneOrMore) && (zoom <= 1.0)) {
+    zoom = 1.0;
+  }
   if ((zoom > PREVIEW_MAX_ZOOM_FACTOR) || (isAtFullZoom() && (zoom < _currentZoomFactor))) {
     emit zoomChanged(_currentZoomFactor);
     return;
@@ -826,35 +830,12 @@ void PreviewWidget::displayOriginalImage()
 
 QSize PreviewWidget::originalImageCropSize()
 {
-  if (_visibleRect != _cachedOriginalImagePosition) {
-    updateCachedOriginalImageCrop();
-  }
-  return QSize(_cachedOriginalImage->width(), _cachedOriginalImage->height());
-}
-
-void PreviewWidget::updateCachedOriginalImageCrop()
-{
-  gmic_list<float> images;
-  gmic_list<char> imageNames;
-  gmic_qt_get_cropped_images(images, imageNames, _visibleRect.x, _visibleRect.y, _visibleRect.w, _visibleRect.h, GmicQt::Active);
-  if (images.size() > 0) {
-    gmic_qt_apply_color_profile(images[0]);
-    _cachedOriginalImage->swap(images[0]);
-    _cachedOriginalImagePosition = _visibleRect;
-  } else {
-    _cachedOriginalImage->assign();
-    _cachedOriginalImagePosition = PreviewRect::Full;
-  }
+  return CroppedActiveLayerProxy::getSize(_visibleRect.x, _visibleRect.y, _visibleRect.w, _visibleRect.h);
 }
 
 void PreviewWidget::getOriginalImageCrop(cimg_library::CImg<float> & image)
 {
-  if (_visibleRect == _cachedOriginalImagePosition) {
-    image = *_cachedOriginalImage;
-    return;
-  }
-  updateCachedOriginalImageCrop();
-  image = *_cachedOriginalImage;
+  CroppedActiveLayerProxy::get(image, _visibleRect.x, _visibleRect.y, _visibleRect.w, _visibleRect.h);
 }
 
 void PreviewWidget::onPreviewParametersChanged()
@@ -908,6 +889,16 @@ void PreviewWidget::setKeypoints(const KeypointList & keypoints)
   update();
 }
 
+void PreviewWidget::setZoomConstraint(ZoomConstraint constraint)
+{
+  _zoomConstraint = constraint;
+}
+
+ZoomConstraint PreviewWidget::zoomConstraint() const
+{
+  return _zoomConstraint;
+}
+
 void PreviewWidget::invalidateSavedPreview()
 {
   _savedPreviewIsValid = false;
